@@ -8,14 +8,20 @@ extern crate clap;
 mod app;
 mod args;
 mod auth;
+mod domain;
 mod http;
+mod infra;
 mod logging;
 mod metrics;
+mod notify;
 mod storage;
 
 use crate::args::FtpsClientAuthType;
 use crate::auth::{DefaultUserProvider, JsonUserProvider};
 
+use crate::app::libunftp_version;
+use crate::domain::{EventDispatcher, FTPEvent, FTPEventPayload};
+use crate::notify::{NotifyingAuthenticator, NotifyingStorageBackend};
 use auth::LookupAuthenticator;
 use clap::ArgMatches;
 use libunftp::{
@@ -25,6 +31,8 @@ use libunftp::{
     Server,
 };
 use slog::*;
+use std::process::Command;
+use std::time::Duration;
 use std::{
     env, fs,
     net::{IpAddr, Ipv4Addr, SocketAddr, ToSocketAddrs},
@@ -42,9 +50,11 @@ use tokio::{
 use unftp_auth_pam as pam;
 use unftp_sbe_gcs::options::AuthMethod;
 
-fn make_auth(m: &clap::ArgMatches) -> Result<Arc<dyn auth_spi::Authenticator<auth::User> + Send + Sync>, String> {
+fn make_auth(
+    m: &clap::ArgMatches,
+) -> Result<Box<dyn auth_spi::Authenticator<auth::User> + Send + Sync + 'static>, String> {
     let mut auth: LookupAuthenticator = match m.value_of(args::AUTH_TYPE) {
-        None | Some("anonymous") => Ok(make_anon_auth()),
+        None | Some("anonymous") => make_anon_auth(),
         Some("pam") => make_pam_auth(m),
         Some("rest") => make_rest_auth(m),
         Some("json") => make_json_auth(m),
@@ -58,11 +68,11 @@ fn make_auth(m: &clap::ArgMatches) -> Result<Arc<dyn auth_spi::Authenticator<aut
         }
         None => Box::new(DefaultUserProvider {}),
     });
-    Ok(Arc::new(auth))
+    Ok(Box::new(auth))
 }
 
-fn make_anon_auth() -> LookupAuthenticator {
-    LookupAuthenticator::new(auth_spi::AnonymousAuthenticator)
+fn make_anon_auth() -> Result<LookupAuthenticator, String> {
+    Ok(LookupAuthenticator::new(auth_spi::AnonymousAuthenticator))
 }
 
 fn make_pam_auth(m: &clap::ArgMatches) -> Result<LookupAuthenticator, String> {
@@ -144,25 +154,43 @@ fn make_json_auth(m: &clap::ArgMatches) -> Result<LookupAuthenticator, String> {
     }
 }
 
-type VfsProducer =
-    Box<dyn (Fn() -> storage::RooterVfs<storage::RestrictingVfs, auth::User, storage::SbeMeta>) + Send + Sync>;
+type VfsProducer = Box<
+    dyn (Fn() -> NotifyingStorageBackend<
+            storage::RooterVfs<storage::RestrictingVfs, auth::User, storage::SbeMeta>,
+            auth::User,
+        >) + Send
+        + Sync,
+>;
 
 // Creates the filesystem storage back-end
-fn fs_storage_backend(log: &Logger, m: &clap::ArgMatches) -> VfsProducer {
+fn fs_storage_backend(
+    log: &Logger,
+    m: &clap::ArgMatches,
+    event_dispatcher: Arc<dyn EventDispatcher<FTPEvent>>,
+) -> VfsProducer {
     let p: PathBuf = m.value_of(args::ROOT_DIR).unwrap().into();
     let sub_log = Arc::new(log.new(o!("module" => "storage")));
     Box::new(move || {
-        storage::RooterVfs::new(storage::RestrictingVfs {
-            delegate: storage::ChoosingVfs {
-                inner: storage::InnerVfs::File(unftp_sbe_fs::Filesystem::new(p.clone())),
-                log: sub_log.clone(),
-            },
-        })
+        notify::NotifyingStorageBackend::new(
+            storage::RooterVfs::new(storage::RestrictingVfs {
+                delegate: storage::ChoosingVfs {
+                    inner: storage::InnerVfs::File(unftp_sbe_fs::Filesystem::new(p.clone())),
+                    log: sub_log.clone(),
+                },
+            }),
+            event_dispatcher.clone(),
+            "",
+            "",
+        )
     })
 }
 
 // Creates the GCS storage back-end
-fn gcs_storage_backend(log: &Logger, m: &clap::ArgMatches) -> Result<VfsProducer, String> {
+fn gcs_storage_backend(
+    log: &Logger,
+    m: &clap::ArgMatches,
+    event_dispatcher: Arc<dyn EventDispatcher<FTPEvent>>,
+) -> Result<VfsProducer, String> {
     let bucket: String = m
         .value_of(args::GCS_BUCKET)
         .ok_or_else(|| format!("--{} is required when using storage type gcs", args::GCS_BUCKET))?
@@ -197,25 +225,54 @@ fn gcs_storage_backend(log: &Logger, m: &clap::ArgMatches) -> Result<VfsProducer
 
     let sub_log = Arc::new(log.new(o!("module" => "storage")));
     Ok(Box::new(move || {
-        storage::RooterVfs::new(storage::RestrictingVfs {
-            delegate: storage::ChoosingVfs {
-                inner: storage::InnerVfs::Cloud(unftp_sbe_gcs::CloudStorage::with_api_base(
-                    base_url.clone(),
-                    bucket.clone(),
-                    root_dir.clone(),
-                    auth_method.clone(),
-                )),
-                log: sub_log.clone(),
-            },
-        })
+        notify::NotifyingStorageBackend::new(
+            storage::RooterVfs::new(storage::RestrictingVfs {
+                delegate: storage::ChoosingVfs {
+                    inner: storage::InnerVfs::Cloud(unftp_sbe_gcs::CloudStorage::with_api_base(
+                        base_url.clone(),
+                        bucket.clone(),
+                        root_dir.clone(),
+                        auth_method.clone(),
+                    )),
+                    log: sub_log.clone(),
+                },
+            }),
+            event_dispatcher.clone(),
+            "xxx",
+            "yyy",
+        )
     }))
 }
 
 // starts the FTP server as a Tokio task.
-fn start_ftp(log: &Logger, root_log: &Logger, m: &clap::ArgMatches) -> Result<(), String> {
+fn start_ftp(
+    log: &Logger,
+    root_log: &Logger,
+    m: &clap::ArgMatches,
+    shutdown: tokio::sync::broadcast::Receiver<()>,
+    done: tokio::sync::mpsc::Sender<()>,
+) -> Result<(), String> {
+    let event_dispatcher = notify::create_event_dispatcher(Arc::new(log.new(o!("module" => "storage"))), m)?;
+
     match m.value_of(args::STORAGE_BACKEND_TYPE) {
-        None | Some("filesystem") => start_ftp_with_storage(log, root_log, m, fs_storage_backend(root_log, m)),
-        Some("gcs") => start_ftp_with_storage(log, root_log, m, gcs_storage_backend(root_log, m)?),
+        None | Some("filesystem") => start_ftp_with_storage(
+            log,
+            root_log,
+            m,
+            fs_storage_backend(root_log, m, event_dispatcher.clone()),
+            event_dispatcher.clone(),
+            shutdown,
+            done,
+        ),
+        Some("gcs") => start_ftp_with_storage(
+            log,
+            root_log,
+            m,
+            gcs_storage_backend(root_log, m, event_dispatcher.clone())?,
+            event_dispatcher.clone(),
+            shutdown,
+            done,
+        ),
         Some(x) => Err(format!("unknown storage back-end type {}", x)),
     }
 }
@@ -259,6 +316,9 @@ fn start_ftp_with_storage<S>(
     root_log: &Logger,
     arg_matches: &ArgMatches,
     storage_backend: Box<dyn (Fn() -> S) + Send + Sync>,
+    event_dispatcher: Arc<dyn EventDispatcher<FTPEvent>>,
+    mut shutdown: tokio::sync::broadcast::Receiver<()>,
+    done: tokio::sync::mpsc::Sender<()>,
 ) -> Result<(), String>
 where
     S: StorageBackend<auth::User> + Send + Sync + 'static,
@@ -310,13 +370,29 @@ where
         (_, false) => SiteMd5::None,
     };
 
-    let mut server = Server::with_authenticator(storage_backend, make_auth(arg_matches)?)
+    let host_name = get_host_name();
+    let instance_name = arg_matches.value_of(args::INSTANCE_NAME).unwrap().to_owned();
+
+    let authenticator = NotifyingAuthenticator::new(
+        make_auth(arg_matches)?,
+        event_dispatcher.clone(),
+        &instance_name,
+        &host_name,
+    );
+
+    let l = log.clone();
+    let mut server = Server::with_authenticator(storage_backend, Arc::new(authenticator))
         .greeting("Welcome to unFTP")
         .passive_ports(start_port..end_port)
         .idle_session_timeout(idle_timeout)
         .logger(root_log.new(o!("lib" => "libunftp")))
         .passive_host(passive_host)
         .sitemd5(md5_setting)
+        .shutdown_indicator(async move {
+            shutdown.recv().await.ok();
+            info!(l, "Shutting down FTP server");
+            libunftp::options::Shutdown::new().grace_period(Duration::from_secs(11))
+        })
         .metrics();
 
     // Setup proxy protocol mode.
@@ -408,7 +484,28 @@ where
         }
     };
 
-    tokio::spawn(server.listen(addr));
+    let log = log.clone();
+    tokio::spawn(async move {
+        if let Err(e) = server.listen(addr).await {
+            error!(log, "FTP server error: {:?}", e)
+        }
+        info!(log, "FTP exiting");
+        drop(done)
+    });
+
+    tokio::spawn(async move {
+        event_dispatcher
+            .dispatch(FTPEvent {
+                source_instance: instance_name,
+                hostname: host_name,
+                payload: FTPEventPayload::Startup {
+                    unftp_version: app::VERSION.to_string(),
+                    libunftp_version: libunftp_version().to_string(),
+                },
+            })
+            .await
+    });
+
     Ok(())
 }
 
@@ -431,24 +528,44 @@ async fn listen_for_signals() -> Result<ExitSignal, String> {
 }
 
 async fn main_task(arg_matches: ArgMatches<'_>, log: &Logger, root_log: &Logger) -> Result<ExitSignal, String> {
+    let (shutdown_sender, http_receiver) = tokio::sync::broadcast::channel(1);
+    let (http_done_sender, mut shutdown_done_received) = tokio::sync::mpsc::channel(1);
+    let ftp_done_sender = http_done_sender.clone();
+
     let ftp_addr: SocketAddr = arg_matches
         .value_of(args::BIND_ADDRESS)
         .unwrap()
         .parse()
         .map_err(|_| "could not parse FTP address")?;
+
     if let Some(addr) = arg_matches.value_of(args::HTTP_BIND_ADDRESS) {
         let addr = String::from(addr);
         let log = log.clone();
         tokio::spawn(async move {
-            if let Err(e) = http::start(&log, &*addr, ftp_addr).await {
+            if let Err(e) = http::start(&log, &*addr, ftp_addr, http_receiver, http_done_sender).await {
                 error!(log, "HTTP Server error: {}", e)
             }
         });
     }
 
-    start_ftp(log, root_log, &arg_matches)?;
+    start_ftp(
+        log,
+        root_log,
+        &arg_matches,
+        shutdown_sender.subscribe(),
+        ftp_done_sender,
+    )?;
 
-    listen_for_signals().await
+    let signal = listen_for_signals().await?;
+    info!(log, "Received signal {}, shutting down...", signal.0);
+
+    drop(shutdown_sender);
+
+    // When every sender has gone out of scope, the recv call
+    // will return with an error. We ignore the error.
+    let _ = shutdown_done_received.recv().await;
+
+    Ok(signal)
 }
 
 fn run(arg_matches: ArgMatches) -> Result<(), String> {
@@ -476,11 +593,24 @@ fn run(arg_matches: ArgMatches) -> Result<(), String> {
     );
 
     let runtime = Runtime::new().map_err(|e| format!("could not construct runtime: {}", e))?;
-    let ExitSignal(signal) = runtime.block_on(main_task(arg_matches, &log, &root_logger))?;
-    info!(log, "Received signal {}, shutting down...", signal);
+    let _ = runtime.block_on(main_task(arg_matches, &log, &root_logger))?;
+    info!(log, "Exiting...");
     Ok(())
 }
 
+fn get_host_name() -> String {
+    if let Ok(host) = env::var("HOST") {
+        return host;
+    }
+    if let Ok(host) = env::var("HOSTNAME") {
+        return host;
+    }
+    match Command::new("hostname").output() {
+        Ok(output) => String::from_utf8_lossy(&output.stdout).replace("\n", ""),
+        Err(_) => "unknown".to_string(),
+    }
+}
+
 fn main() {
     let tmp_dir = env::temp_dir();
     let tmp_dir = tmp_dir.as_path().to_str().unwrap();
